@@ -149,7 +149,7 @@ template <typename K> SPIRVCapVec getCapability(K Key) {
 #define ADD_VEC_INIT(Cap, ...)                                                 \
   {                                                                            \
     SPIRVCapabilityKind C[] = __VA_ARGS__;                                     \
-    SPIRVCapVec V(C, C + sizeof(C) / sizeof(C[0]));                            \
+    const SPIRVCapVec V(C, C + sizeof(C) / sizeof(C[0]));                      \
     add(Cap, V);                                                               \
   }
 
@@ -211,23 +211,6 @@ template <> inline void SPIRVMap<SPIRVCapabilityKind, SPIRVCapVec>::init() {
                {CapabilityCooperativeMatrixKHR});
   ADD_VEC_INIT(internal::CapabilityCooperativeMatrixCheckedInstructionsINTEL,
                {CapabilityCooperativeMatrixKHR});
-  ADD_VEC_INIT(CapabilityBFloat16DotProductKHR, {CapabilityBFloat16TypeKHR});
-  ADD_VEC_INIT(CapabilityBFloat16CooperativeMatrixKHR,
-               {CapabilityBFloat16TypeKHR, CapabilityCooperativeMatrixKHR});
-  ADD_VEC_INIT(internal::CapabilityBFloat16ArithmeticINTEL,
-               {CapabilityBFloat16TypeKHR});
-  ADD_VEC_INIT(internal::CapabilityAtomicInt16CompareExchangeINTEL,
-               {CapabilityInt16});
-  ADD_VEC_INIT(internal::CapabilityInt16AtomicsINTEL,
-               {internal::CapabilityAtomicInt16CompareExchangeINTEL});
-  ADD_VEC_INIT(internal::CapabilityAtomicBFloat16LoadStoreINTEL,
-               {CapabilityBFloat16TypeKHR});
-  ADD_VEC_INIT(internal::CapabilityAtomicBFloat16AddINTEL,
-               {CapabilityBFloat16TypeKHR});
-  ADD_VEC_INIT(internal::CapabilityAtomicBFloat16MinMaxINTEL,
-               {CapabilityBFloat16TypeKHR});
-  ADD_VEC_INIT(CapabilityInt4CooperativeMatrixINTEL,
-               {CapabilityInt4TypeINTEL, CapabilityCooperativeMatrixKHR});
 }
 
 template <> inline void SPIRVMap<SPIRVExecutionModelKind, SPIRVCapVec>::init() {
@@ -290,8 +273,8 @@ template <> inline void SPIRVMap<SPIRVExecutionModeKind, SPIRVCapVec>::init() {
                {CapabilityFloatingPointModeINTEL});
   ADD_VEC_INIT(ExecutionModeSharedLocalMemorySizeINTEL,
                {CapabilityVectorComputeINTEL});
-  ADD_VEC_INIT(ExecutionModeRegisterMapInterfaceINTEL,
-               {CapabilityFPGAKernelAttributesv2INTEL});
+  ADD_VEC_INIT(internal::ExecutionModeFastCompositeKernelINTEL,
+               {internal::CapabilityFastCompositeINTEL});
   ADD_VEC_INIT(internal::ExecutionModeStreamingInterfaceINTEL,
                {CapabilityFPGAKernelAttributesINTEL});
   ADD_VEC_INIT(ExecutionModeNamedBarrierCountINTEL,
@@ -464,6 +447,8 @@ template <> inline void SPIRVMap<Decoration, SPIRVCapVec>::init() {
   ADD_VEC_INIT(DecorationAliasScopeINTEL,
                {CapabilityMemoryAccessAliasingINTEL});
   ADD_VEC_INIT(DecorationNoAliasINTEL, {CapabilityMemoryAccessAliasingINTEL});
+  ADD_VEC_INIT(internal::DecorationCallableFunctionINTEL,
+               {internal::CapabilityFastCompositeINTEL});
   ADD_VEC_INIT(DecorationMediaBlockIOINTEL, {CapabilityVectorComputeINTEL});
   ADD_VEC_INIT(DecorationStallEnableINTEL,
                {CapabilityFPGAClusterAttributesINTEL});
